@@ -505,7 +505,7 @@ const Bookmarks = Module("bookmarks", {
                 let jumps = [[idx == sh.index ? ">" : "",
                               Math.abs(idx - sh.index),
                               val.title,
-                              let(url=val.URI.spec) xml`<a highlight="URL" href=${url}>${url}</a>`]
+                              xml`<a highlight="URL" href=${val.URI.spec}>${val.URI.spec}</a>`]
                               for ([idx, val] in Iterator(sh))];
                 let list = template.tabular([{ header: "Jump", style: "color: red", colspan: 2 },
                     { header: "", style: "text-align: right", highlight: "Number" },
@@ -599,19 +599,28 @@ const Bookmarks = Module("bookmarks", {
             "Delete a bookmark",
             function (args) {
                 if (args.bang) {
-                    commandline.input("This will delete all bookmarks. Would you like to continue? (yes/[no]) ",
-                        function (resp) {
-                            if (resp && resp.match(/^y(es)?$/i)) {
-                                bookmarks._cache.bookmarks.forEach(function (bmark) { services.get("bookmarks").removeItem(bmark.id); });
-                                liberator.echomsg("All bookmarks deleted");
-                            }
-                        });
+
+                    function deleteAll() {
+                        bookmarks._cache.bookmarks.forEach(function (bmark) { services.get("bookmarks").removeItem(bmark.id); });
+                        liberator.echomsg("All bookmarks deleted");
+                    }
+
+                    if (commandline.silent)
+                        deleteAll();
+                    else {
+                        commandline.input("This will delete all bookmarks. Would you like to continue? (yes/[no]) ",
+                            function (resp) {
+                                if (resp && resp.match(/^y(es)?$/i)) {
+                                    deleteAll();
+                                }
+                            });
+                    }
                 }
                 else {
                     let url = args.string || buffer.URL;
                     let deletedCount = bookmarks.remove(url);
 
-                    liberator.echomsg("Deleted " + deletedCount + " bookmark(s) with url: " + url.quote());
+                    liberator.echomsg("Deleted " + deletedCount + " bookmark(s) with url: " + JSON.stringify(url));
                 }
 
             },
