@@ -208,7 +208,13 @@ const CompletionContext = Class("CompletionContext", {
                     __proto__: item
                 }));
             });
-            return { start: minStart, items: util.Array.flatten(items), longestSubstring: this.longestAllSubstring };
+            return { start: minStart, items: util.Array.flatten(items),
+                get longestSubstring() {
+                    var value = this.longestAllSubstring;
+                    Object.defineProperty(this, "longestSubstring", { value: value});
+                    return value;
+                }
+            };
         }
         catch (e) {
             liberator.echoerr(e);
@@ -507,8 +513,8 @@ const CompletionContext = Class("CompletionContext", {
 
     getText: function getText(item) {
         let text = item[self.keys["text"]];
-        if (self.quote)
-            return self.quote(text);
+        //if (self.quote)
+            //return self.quote(text);
         return text;
     },
 
