@@ -62,36 +62,6 @@ const Editor = Module("editor", {
         return text.substring(e.selectionStart, e.selectionEnd);
     },
 
-    pasteClipboard: function () {
-        if (liberator.has("Windows")) {
-            this.executeCommand("cmd_paste");
-            return;
-        }
-
-        // FIXME: #93 (<s-insert> in the bottom of a long textarea bounces up)
-        let elem = liberator.focus;
-
-        if (elem.setSelectionRange && util.readFromClipboard()) {
-            // readFromClipboard would return 'undefined' if not checked
-            // dunno about .setSelectionRange
-            // This is a hacky fix - but it works.
-            let curTop = elem.scrollTop;
-            let curLeft = elem.scrollLeft;
-
-            let rangeStart = elem.selectionStart; // caret position
-            let rangeEnd = elem.selectionEnd;
-            let tempStr1 = elem.value.substring(0, rangeStart);
-            let tempStr2 = util.readFromClipboard();
-            let tempStr3 = elem.value.substring(rangeEnd);
-            elem.value = tempStr1 + tempStr2 + tempStr3;
-            elem.selectionStart = rangeStart + tempStr2.length;
-            elem.selectionEnd = elem.selectionStart;
-
-            elem.scrollTop = curTop;
-            elem.scrollLeft = curLeft;
-        }
-    },
-
     // count is optional, defaults to 1
     executeCommand: function (cmd, count) {
         let controller = Editor.getController(cmd);
@@ -310,11 +280,11 @@ const Editor = Module("editor", {
             nsEditor instanceof Ci.nsIHTMLEditor;
         }
         else {
-	    if (field !== undefined) {
-		    textBox = field; // if I reply 'yes', textBox.value == 'yes', so get the real value
-	    } else {
-		    textBox = liberator.focus;
-	    }
+            if (field !== undefined) {
+                textBox = field; // if I reply 'yes', textBox.value == 'yes', so get the real value
+            } else {
+                textBox = liberator.focus;
+            }
         }
 
         if (!forceEditing && textBox && textBox.type == "password") {
@@ -367,8 +337,10 @@ const Editor = Module("editor", {
                     lastUpdate = Date.now();
 
                     let val = tmpfile.read();
-                    if (textBox)
+                    if (textBox) {
                         textBox.value = val;
+                        textBox.dispatchEvent(new InputEvent('input'));
+                    }
                     else if (nsEditor) {
                         let wholeDocRange = nsEditor.document.createRange();
                         let rootNode = nsEditor.rootElement.QueryInterface(Ci.nsIDOMNode);
@@ -446,6 +418,9 @@ const Editor = Module("editor", {
         if (!textbox)
             return false;
         let text      = textbox.value;
+        if (typeof text !== "string")
+            return false;
+
         let currStart = textbox.selectionStart;
         let currEnd   = textbox.selectionEnd;
         let foundWord = text.substring(0, currStart).replace(/.*[\s\n]/gm, '').match(RegExp('(' + abbreviations._match + ')$'));
@@ -666,17 +641,18 @@ const Editor = Module("editor", {
             ["<C-End>"], "Move cursor to end of text field",
             function () { editor.executeCommand("cmd_moveBottom", 1); });*/
 
-        mappings.add(myModes,
-            ["<S-Insert>"], "Insert clipboard/selection",
-            function () { editor.pasteClipboard(); });
-
         mappings.add(modes.getCharModes("i"),
             ["<C-i>"], "Edit text field with an external editor",
             function () { editor.editFieldExternally(); });
 
         mappings.add([modes.INSERT],
             ["<C-t>"], "Edit text field in Vi mode",
-            function () { liberator.mode = modes.TEXTAREA; });
+            function () {
+                let edtr = Editor.getEditor();
+                if (edtr.setSelectionRange)
+                    edtr.setSelectionRange(edtr.selectionStart - 1, edtr.selectionStart - 1);
+                liberator.mode = modes.TEXTAREA;
+            });
 
         mappings.add([modes.INSERT],
             ["<Space>", "<Return>"], "Expand insert mode abbreviation",
@@ -809,7 +785,7 @@ const Editor = Module("editor", {
             function (count, arg) {
                 let pos = editor.findCharForward(arg, count);
                 if (pos >= 0)
-                    editor.moveToPosition(pos, true, liberator.mode == modes.VISUAL);
+                    editor.moveToPosition(pos - 1, true, liberator.mode == modes.VISUAL);
             },
             { arg: true, count: true });
 
@@ -827,7 +803,7 @@ const Editor = Module("editor", {
             function (count, arg) {
                 let pos = editor.findCharForward(arg, count);
                 if (pos >= 0)
-                    editor.moveToPosition(pos - 1, true, liberator.mode == modes.VISUAL);
+                    editor.moveToPosition(pos - 2, true, liberator.mode == modes.VISUAL);
             },
             { arg: true, count: true });
 
