@@ -24,7 +24,7 @@ const Events = Module("events", {
 
         this.sessionListeners = [];
 
-        this._macros = storage.newMap("macros", { store: true, privateData: true });
+        this._macros = storage.newMap("macros", { store: true });
 
         // NOTE: the order of ["Esc", "Escape"] or ["Escape", "Esc"]
         //       matters, so use that string as the first item, that you
@@ -478,6 +478,8 @@ const Events = Module("events", {
      *
      * @param {Event} event
      * @returns {string}
+     *
+     * @see http://unixpapa.com/js/key.html
      */
     toString: function (event) {
         if (!event)
@@ -497,40 +499,63 @@ const Events = Module("events", {
             modifier += "M-";
 
         if (/^key/.test(event.type)) {
-            let charCode = event.type == "keyup" ? 0 : event.charCode;
+            let charCode = (event.type == "keydown" || event.type == "keyup") ? 0 : event.charCode;
             if (charCode == 0) {
                 if (event.shiftKey)
                     modifier += "S-";
 
                 if (event.keyCode in this._code_key)
                     key = this._code_key[event.keyCode];
-            }
-            // [Ctrl-Bug] special handling of mysterious <C-[>, <C-\\>, <C-]>, <C-^>, <C-_> bugs (OS/X)
-            //            (i.e., cntrl codes 27--31)
-            // ---
-            // For more information, see:
-            //     [*] Vimp FAQ: http://vimperator.org/trac/wiki/Vimperator/FAQ#WhydoesntC-workforEscMacOSX
-            //     [*] Referenced mailing list msg: http://www.mozdev.org/pipermail/vimperator/2008-May/001548.html
-            //     [*] Mozilla bug 416227: event.charCode in keypress handler has unexpected values on Mac for Ctrl with chars in "[ ] _ \"
-            //         https://bugzilla.mozilla.org/show_bug.cgi?query_format=specific&order=relevance+desc&bug_status=__open__&id=416227
-            //     [*] Mozilla bug 432951: Ctrl+'foo' doesn't seem same charCode as Meta+'foo' on Cocoa
-            //         https://bugzilla.mozilla.org/show_bug.cgi?query_format=specific&order=relevance+desc&bug_status=__open__&id=432951
-            // ---
-            //
-            // The following fixes are only activated if liberator.has("MacUnix").
-            // Technically, they prevent mappings from <C-Esc> (and
-            // <C-C-]> if your fancy keyboard permits such things<?>), but
-            // these <C-control> mappings are probably pathological (<C-Esc>
-            // certainly is on Windows), and so it is probably
-            // harmless to remove the has("MacUnix") if desired.
-            //
-            else if (liberator.has("MacUnix") && event.ctrlKey && charCode >= 27 && charCode <= 31) {
-                if (charCode == 27) { // [Ctrl-Bug 1/5] the <C-[> bug
-                    key = "Esc";
-                    modifier = modifier.replace("C-", "");
+
+                // [Ctrl-Bug] special handling of mysterious <C-[>, <C-\\>, <C-]>, <C-^>, <C-_> <C--> bugs (OS/X)
+                // ---
+                // For more information, see:
+                //     [*] Vimp FAQ: http://vimperator.org/trac/wiki/Vimperator/FAQ#WhydoesntC-workforEscMacOSX
+                //     [*] Referenced mailing list msg: http://www.mozdev.org/pipermail/vimperator/2008-May/001548.html
+                //     [*] Mozilla bug 416227: event.charCode in keypress handler has unexpected values on Mac for Ctrl with chars in "[ ] _ \"
+                //         https://bugzilla.mozilla.org/show_bug.cgi?query_format=specific&order=relevance+desc&bug_status=__open__&id=416227
+                //     [*] Mozilla bug 432951: Ctrl+'foo' doesn't seem same charCode as Meta+'foo' on Cocoa
+                //         https://bugzilla.mozilla.org/show_bug.cgi?query_format=specific&order=relevance+desc&bug_status=__open__&id=432951
+                // ---
+                //
+                // The following fixes are only activated if liberator.has("MacUnix").
+                // Technically, they prevent mappings from <C-Esc> (and
+                // <C-C-]> if your fancy keyboard permits such things<?>), but
+                // these <C-control> mappings are probably pathological (<C-Esc>
+                // certainly is on Windows), and so it is probably
+                // harmless to remove the has("MacUnix") if desired.
+
+                if (liberator.has("MacUnix") && event.ctrlKey) {
+                    switch (event.keyCode) {
+                    case 219:   // <C-[>
+                        key = "Esc";
+                        modifier = modifier.replace("C-", "");
+                        break;
+                    case 220:   // <C-\\>
+                        key = "\\";
+                        break;
+                    case 221:   // <C-]>
+                        key = "]";
+                        break;
+                    case 173:   // <C-_> and <C-->
+                        key = event.shiftKey ? "_" : "-";
+                        modifier = modifier.replace("S-", "");
+                        break;
+                    case 54:   // <C-^>
+                        key = "^";
+                        modifier = modifier.replace("S-", "");
+                        break;
+                    default:
+                        break;
+                    }
+                }
+
+                if (!key) { // For alphabetical caracters
+                    key = String.fromCharCode(event.keyCode); // Gives by default the uppercased version for keydown/keyup of normal keys
+                    if (!event.shiftKey)
+                        key = key.toLowerCase();
+                    return key; // Always the "pure" key without any <..> around them or without any modifiers
                 }
-                else // [Ctrl-Bug 2,3,4,5/5] the <C-\\>, <C-]>, <C-^>, <C-_> bugs
-                    key = String.fromCharCode(charCode + 64);
             }
             // a normal key like a, b, c, 0, etc.
             else if (charCode > 0) {
@@ -598,6 +623,14 @@ const Events = Module("events", {
      */
     isCancelKey: function (key) key == "<Esc>" || key == "<C-[>" || key == "<C-c>",
 
+    /**
+     * Whether <b>key</b> is a key code defined to go back to NORMAL mode
+     *
+     * @param {string} key The key code to test.
+     * @returns {boolean}
+     */
+    isEscapeKey: function (key) key == "<Esc>" || key == "<C-[>",
+
     /**
      * Waits for the current buffer to successfully finish loading. Returns
      * true for a successful page load otherwise false.
@@ -826,8 +859,6 @@ const Events = Module("events", {
     // the commandline has focus
     // TODO: ...help me...please...
     onKeyPress: function (event) {
-        function isEscapeKey(key) key == "<Esc>" || key == "<C-[>";
-
         function killEvent() {
             event.preventDefault();
             event.stopPropagation();
@@ -927,7 +958,7 @@ const Events = Module("events", {
 
             // TODO: handle middle click in content area
 
-            if (!isEscapeKey(key)) {
+            if (!this.isEscapeKey(key)) {
                 // custom mode...
                 if (liberator.mode == modes.CUSTOM) {
                     plugins.onEvent(event);
@@ -987,7 +1018,7 @@ const Events = Module("events", {
                 this._input.buffer = "";
                 let map = this._input.pendingArgMap;
                 this._input.pendingArgMap = null;
-                if (!isEscapeKey(key)) {
+                if (!this.isEscapeKey(key)) {
                     if (modes.isReplaying && !this.waitForPageLoad())
                         return;
                     map.execute(null, this._input.count, key);
@@ -1004,7 +1035,7 @@ const Events = Module("events", {
                     this._input.pendingArgMap = map;
                 }
                 else if (this._input.pendingMotionMap) {
-                    if (!isEscapeKey(key))
+                    if (!this.isEscapeKey(key))
                         this._input.pendingMotionMap.execute(candidateCommand, this._input.count, null);
                     this._input.pendingMotionMap = null;
                 }
@@ -1037,7 +1068,7 @@ const Events = Module("events", {
                 this._input.pendingMotionMap = null;
                 this._input.pendingMap = null;
 
-                if (!isEscapeKey(key)) {
+                if (!this.isEscapeKey(key)) {
                     // allow key to be passed to the host app if we can't handle it
                     stop = false;
 
@@ -1065,11 +1096,38 @@ const Events = Module("events", {
         }
     },
 
-    // this is need for sites like msn.com which focus the input field on keydown
     onKeyUpOrDown: function (event) {
+        // Always let the event be handled by the webpage/Firefox for certain modes
         if (modes.passNextKey || modes.passAllKeys || modes.isMenuShown || Events.isInputElemFocused())
             return;
-        event.stopPropagation();
+
+        // Many sites perform (useful) actions on keydown.
+        // Let's keep the most common ones unless we have a mapping for that
+        let key = events.toString(event);
+        if (event.type == "keydown" && this.isEscapeKey(key)) {
+            this.onEscape(); // We do our Escape handling here, as the on "onKeyPress" may not always work if websites override the keydown event
+            event.stopPropagation();
+            return;
+        }
+
+        let url = typeof(buffer) != "undefined" ? buffer.URL : "";
+        let inputStr = this._input.buffer + key;
+        let countStr = inputStr.match(/^[1-9][0-9]*|/)[0];
+        let candidateCommand = inputStr.substr(countStr.length);
+        let map = mappings[event.noremap ? "getDefault" : "get"](liberator.mode, candidateCommand, url);
+        if (map) {
+            event.stopPropagation();
+            return;
+        }
+
+        // Or we have a mapping starting with that key
+        let candidates = mappings.getCandidates(liberator.mode, candidateCommand, url);
+        if (candidates.length > 0) { // We want to handle that key ourselves
+            event.stopPropagation();
+            return;
+        }
+
+        // liberator.echo ("key: " + key + "\nkeycode: " + event.keyCode + "\nchar: " + event.charCode + "\ntype: " + event.type + "\nwhich: " + event.which);
     },
 
     onPopupShown: function (event) {
@@ -1106,10 +1164,14 @@ const Events = Module("events", {
 }, {
     isInputElemFocused: function () {
         let elem = liberator.focus;
+        if (!elem) {
+            return false;
+        }
         return ((elem instanceof HTMLInputElement && !/image/.test(elem.type)) ||
                  elem instanceof HTMLTextAreaElement ||
                  elem instanceof HTMLObjectElement ||
-                 elem instanceof HTMLEmbedElement);
+                 elem instanceof HTMLEmbedElement ||
+                 (elem && elem.contentEditable === "true"));
     }
 }, {
     commands: function () {
