@@ -272,7 +272,7 @@ const File = Class("File", {
         // TODO: Vim expands paths twice, once before checking for ~, once
         // after, but doesn't document it. Is this just a bug? --Kris
         path = expand(path);
-        return path.replace("/", IO.PATH_SEP, "g");
+        return path.replace(/\//g, IO.PATH_SEP);
     },
 
     getPathsFromPathList: function (list) {
@@ -330,9 +330,6 @@ const IO = Module("io", {
         this._lastRunCommand = ""; // updated whenever the users runs a command with :!
         this._scriptNames = [];
 
-        // XXX: nsIDownloadManager is deprecated on Firefox 26
-        // FIXME: need to listen to download state ? -- teramako
-        // FIXME: need to adapt to Download.jsm instead of nsIDownloadManager
         if (services.get("vc").compare(Application.version, "26.0a1") < 0) {
             this.downloadListener = {
                 onDownloadStateChange: function (state, download) {
@@ -889,7 +886,7 @@ lookup:
                     }
 
                     if (!found)
-                        liberator.echoerr("Can't find directory " + arg.quote() + " in cdpath\n" + "Command failed");
+                        liberator.echoerr("Can't find directory " + JSON.stringify(arg) + " in cdpath\n" + "Command failed");
                 }
             }, {
                 argCount: "?",
@@ -1020,14 +1017,23 @@ lookup:
 
         completion.charset = function (context) {
             context.anchored = false;
-            context.generate = function () {
-                let names = util.Array(
-                    "more1 more2 more3 more4 more5 static".split(" ").map(function (key)
-                        options.getPref("intl.charsetmenu.browser." + key).split(', '))
-                ).flatten().uniq();
-                let bundle = document.getElementById("liberator-charset-bundle");
-                return names.map(function (name) [name, bundle.getString(name.toLowerCase() + ".title")]);
-            };
+            if (services.get("vc").compare(Application.version, "32") < 0) {
+                context.generate = function () {
+                    let names = util.Array(
+                        "more1 more2 more3 more4 more5 static".split(" ").map(function (key)
+                            options.getPref("intl.charsetmenu.browser." + key).split(', '))
+                    ).flatten().uniq();
+                    let bundle = document.getElementById("liberator-charset-bundle");
+                    return names.map(function (name) [name, bundle.getString(name.toLowerCase() + ".title")]);
+                };
+            }
+            else {
+                context.generate = function () {
+                    let {CharsetMenu} = Cu.import("resource://gre/modules/CharsetMenu.jsm", {});
+                    let data = CharsetMenu.getData();
+                    return data.pinnedCharsets.concat(data.otherCharsets).map(function (o) [o.value, o.label]);
+                };
+            }
         };
 
         completion.directory = function directory(context, full) {
