@@ -5,7 +5,10 @@
 
 (function () {
     const modules = {};
-    const BASE = "liberator://template/chrome://liberator/content/"
+    // TODO: FIXME when minVersion is greater than 34
+    var {isSupport: TemplateIsSupport} = Components.utils.import("resource://liberator/CheckTemplate.jsm", {});
+    const BASE = TemplateIsSupport ?
+        "chrome://liberator/content/" : "liberator://template/chrome://liberator/content/";
 
     modules.modules = modules;
 
@@ -30,8 +33,8 @@
 
     let prefix = [BASE];
 
-    //Cu.import("resource://liberator/template-tag.js", modules);
-    loader.loadSubScript("resource://liberator/template-tag.js", modules);
+    // TODO: FIXME when minVersion is greater than 34
+    Components.utils.import("resource://liberator/template-tag" + (TemplateIsSupport ? ".js" : "-old.js"), modules);
 
     // TODO: This list is much too long, we should try to minimize
     // the number of required components for easier porting to new applications
