@@ -130,7 +130,7 @@ const Liberator = Module("liberator", {
     forceNewPrivateWindow: false,
 
     /** @property {string} The Liberator version string. */
-    version: "3.8.2 (created: 2014/02/13 23:52:43)", // these VERSION and DATE tokens are replaced by the Makefile
+    version: "3.10.1 (created: 2015/08/25 00:41:59)", // these VERSION and DATE tokens are replaced by the Makefile
 
     /**
      * @property {Object} The map of command-line options. These are
@@ -254,6 +254,25 @@ const Liberator = Module("liberator", {
         window.dump(msg.replace(/^./gm, ("config" in modules && config.name.toLowerCase()) + ": $&"));
     },
 
+    isPrivateWindow: function () {
+        return window.QueryInterface(Ci.nsIInterfaceRequestor)
+                     .getInterface(Ci.nsIWebNavigation)
+                     .QueryInterface(Ci.nsILoadContext)
+                     .usePrivateBrowsing;
+    },
+
+    windowID: function() {
+        return window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
+                     .getInterface(Components.interfaces.nsIDOMWindowUtils)
+                     .outerWindowID;
+
+    },
+
+    storeName: function(mode, isPrivate) {
+        let prefix = isPrivate ? "private-" + this.windowID() + "-" : "";
+        return prefix + "history-" + mode ;
+    },
+
     /**
      * Outputs a plain message to the command line.
      *
@@ -332,6 +351,7 @@ const Liberator = Module("liberator", {
      *     should be loaded.
      */
     loadScript: function (uri, context) {
+        // TODO: delete me when minVersion is greater than 34
         if (options.expandtemplate) {
             var prefix = "liberator://template/";
             if (uri.lastIndexOf(prefix, 0) === -1)
@@ -345,6 +365,7 @@ const Liberator = Module("liberator", {
             if (!context)
                 context = userContext;
 
+            // TODO: delete me when minVersion is greater than 34
             if (options.expandtemplate) {
                 var obj = new Object;
                 Cu.import("resource://liberator/template.js", obj);
@@ -524,20 +545,36 @@ const Liberator = Module("liberator", {
         let fileMap = services.get("liberator:").FILE_MAP;
         let overlayMap = services.get("liberator:").OVERLAY_MAP;
 
+        // XXX: util.httpGet is very heavy on startup. (Fx32)
+        function httpGet(url) {
+            var channel = services.get("io").newChannelFromURI(makeURI(url));
+            try {
+                var stream = channel.open();
+            } catch (ex) {
+                return null;
+            }
+
+            var ps = new DOMParser;
+            var res = ps.parseFromStream(stream, "utf-8", stream.available(), "text/xml");
+            stream.close();
+            return {responseXML: res};
+        }
+
         // Left as an XPCOM instantiation so it can easilly be moved
         // into XPCOM code.
         function XSLTProcessor(sheet) {
             let xslt = Cc["@mozilla.org/document-transformer;1?type=xslt"].createInstance(Ci.nsIXSLTProcessor);
-            xslt.importStylesheet(util.httpGet(sheet).responseXML);
+            xslt.importStylesheet(httpGet(sheet).responseXML);
             return xslt;
         }
 
         // Find help and overlay files with the given name.
         function findHelpFile(file) {
             let result = [];
+
             for (let namespace of namespaces) {
                 let url = ["chrome://", namespace, "/locale/", file, ".xml"].join("");
-                let res = util.httpGet(url);
+                let res = httpGet(url);
                 if (res) {
                     if (res.responseXML.documentElement.localName == "document")
                         fileMap[file] = url;
@@ -577,7 +614,7 @@ const Liberator = Module("liberator", {
         const ps = new DOMParser;
         const encoder = Cc["@mozilla.org/layout/documentEncoder;1?type=text/xml"].getService(Ci.nsIDocumentEncoder);
         encoder.init(document, "text/xml", 0);
-        body = xml.map([con for ([,con] in Iterator(plugins.contexts))], function (context) {
+        var body = xml.map([con for ([,con] in Iterator(plugins.contexts))], function (context) {
             try { // debug
             var info = context.INFO;
             if (!info) return "";
@@ -611,7 +648,7 @@ const Liberator = Module("liberator", {
             </document>`.toString();
         fileMap["plugins"] = function () ['text/xml;charset=UTF-8', help];
 
-        addTags("plugins", util.httpGet("liberator://help/plugins").responseXML);
+        addTags("plugins", httpGet("liberator://help/plugins").responseXML);
     },
 
     /**
@@ -628,7 +665,7 @@ const Liberator = Module("liberator", {
             if (helpFile in services.get("liberator:").FILE_MAP)
                 liberator.open("liberator://help/" + helpFile, { from: "help" });
             else
-                liberator.echomsg("Sorry, help file " + helpFile.quote() + " not found");
+                liberator.echomsg("Sorry, help file " + JSON.stringify(helpFile) + " not found");
             return;
         }
 
@@ -675,7 +712,8 @@ const Liberator = Module("liberator", {
         }
 
         liberator.log('Searching for "plugin/**/*.{js,vimp}" in "'
-                            + [dir.path.replace(/.plugin$/, "") for (dir of dirs)].join(",") + '"');
+                            + dirs.map(function(dir) { return dir.path.replace(/.plugins$/, ''); }).join(',')
+                            + '"');
 
         dirs.forEach(function (dir) {
             liberator.log("Searching for \"" + (dir.path + "/**/*.{js,vimp}") + "\"", 3);
@@ -1153,10 +1191,7 @@ const Liberator = Module("liberator", {
                         win.setAttribute("titlemodifier_normal", value);
                         win.setAttribute("titlemodifier_privatebrowsing", value + suffix);
 
-                        if (window.QueryInterface(Ci.nsIInterfaceRequestor)
-                                  .getInterface(Ci.nsIWebNavigation)
-                                  .QueryInterface(Ci.nsILoadContext)
-                                  .usePrivateBrowsing) {
+                        if (liberator.isPrivateWindow()) {
                             win.setAttribute("titlemodifier", value + suffix);
                         }
                         else
@@ -1270,9 +1305,11 @@ const Liberator = Module("liberator", {
                 },
                 validator: function (value) {
                     let toolbars = config.toolbars || {};
-                    // "ne" is a simple hack, since in the next line val.replace() makes "ne" out from "none"
-                    let values = ["all", "ne"].concat([toolbar for each([toolbar, ] in Iterator(toolbars))]);
-                    return value.every(function(val) values.indexOf(val.replace(/^(no|inv)/, "")) >= 0);
+                    // "ne" is a simple hack, since val.replace(), below, makes "ne" out from "none"
+                    let opts = ['all', 'ne'].concat(Object.keys(toolbars));
+                    return value.every(function(val) {
+                        return opts.indexOf(val.replace(/^(no|inv)/, "")) >= 0;
+                    });
                 }
             });
 
@@ -1339,7 +1376,7 @@ const Liberator = Module("liberator", {
                     liberator.echoerr("Invalid argument: " + arg);
                 }
                 catch (e) {
-                    liberator.echoerr("Error opening " + arg.quote() + ": " + e);
+                    liberator.echoerr("Error opening " + JSON.stringify(arg) + ": " + e);
                 }
             }, {
                 argCount: "1",
@@ -1717,9 +1754,9 @@ const Liberator = Module("liberator", {
                     return void liberator.echoerr("No usage information for: " + args[0]);
 
                 if (args[0])
-                    var usage = template.genericOutput(config.name + " Usage", usage[args[0]]());
+                    usage = template.genericOutput(config.name + " Usage", usage[args[0]]());
                 else
-                    var usage = template.genericOutput(config.name + " Usage", xml`${ usage["mappings"]() }<br/>${ usage["commands"]() }<br/>${ usage["options"]()}`);
+                    usage = template.genericOutput(config.name + " Usage", xml`${ usage["mappings"]() }<br/>${ usage["commands"]() }<br/>${ usage["options"]()}`);
                 liberator.echo(usage, commandline.FORCE_MULTILINE);
             }, {
                 argCount: "?",
