@@ -123,11 +123,11 @@ const Option = Class("Option", {
         else
             scope = this.scope;
 
+        let value;
         // Options with a custom getter are always responsible for returning a meaningful value
         if (this.getter)
             return liberator.trapErrors(this.getter, this, value);
 
-        let value;
         if (liberator.has("tabs") && (scope & Option.SCOPE_LOCAL))
             value = tabs.options[this.name];
         if ((scope & Option.SCOPE_GLOBAL) && (value == undefined))
@@ -197,7 +197,7 @@ const Option = Class("Option", {
      * Returns whether the specified <b>values</b> are valid for this option.
      * @see Option#validator
      */
-    isValidValue: function (values) this.validator(values),
+    isValidValue: function (vs) this.validator(vs),
 
     /**
      * Resets the option to its default value.
@@ -532,7 +532,7 @@ const Options = Module("options", {
 
         if (option.name in this._optionHash) {
             // never replace for now
-            liberator.echomsg("Option '" + names[0].quote() + "' already exists, NOT replacing existing option.");
+            liberator.echomsg("Option '" + JSON.stringify(names[0]) + "' already exists, NOT replacing existing option.");
             return false;
         }
 
@@ -716,7 +716,7 @@ const Options = Module("options", {
      *     any of the options's names.
      */
     remove: function (name) {
-        for each (let option in this._optionHash) {
+        for (let option of this._optionHash) {
             if (option.hasName(name))
                 delete this._optionHash[option.name];
         }
