@@ -12,7 +12,7 @@ const QuickMarks = Module("quickmarks", {
     requires: ["config", "storage"],
 
     init: function () {
-        this._qmarks = storage.newMap("quickmarks", { store: true, privateData: true });
+        this._qmarks = storage.newMap("quickmarks", { store: true });
     },
 
     /**
