@@ -261,7 +261,7 @@ function Highlights(name, store) {
         }
 
         let css = newStyle.replace(/(?:!\s*important\s*)?(?:;?\s*$|;)/g, "!important;")
-                          .replace(";!important;", ";", "g"); // Seeming Spidermonkey bug
+                          .replace(/;!important;/g, ";"); // Seeming Spidermonkey bug
         if (!/^\s*(?:!\s*important\s*)?;*\s*$/.test(css)) {
             css = style.selector + " { " + css + " }";
 
@@ -339,9 +339,9 @@ function Styles(name, store) {
     const storage = modules.storage;
     const ios = services.get("io");
     const sss = Cc["@mozilla.org/content/style-sheet-service;1"].getService(Ci.nsIStyleSheetService);
-    const namespace = "@namespace html " + XHTML.uri.quote() + ";\n" +
-                      "@namespace xul " + XUL.uri.quote() + ";\n" +
-                      "@namespace liberator " + NS.uri.quote() + ";\n";
+    const namespace = "@namespace html " + JSON.stringify(XHTML.uri) + ";\n" +
+                      "@namespace xul " + JSON.stringify(XUL.uri) + ";\n" +
+                      "@namespace liberator " + JSON.stringify(NS.uri) + ";\n";
 
     const Sheet = Struct("name", "id", "sites", "css", "system", "agent");
     Sheet.prototype.__defineGetter__("fullCSS", function wrapCSS() {
@@ -401,10 +401,17 @@ function Styles(name, store) {
     this.addSheet = function (system, name, filter, css, agent) {
         let sheets = system ? systemSheets : userSheets;
         let names = system ? systemNames : userNames;
-        if (name && name in names)
+        let sites = filter.split(",").filter(util.identity);
+        if (name && name in names) {
+            let s = names[name];
+            if (s.css === css && s.sites.join(",") === sites.join(",")) {
+                if (!s.enabled) s.enabled = true;
+                return null;
+            }
             this.removeSheet(system, name);
+        }
 
-        let sheet = Sheet(name, id++, filter.split(",").filter(util.identity), String(css), system, agent);
+        let sheet = Sheet(name, id++, sites, String(css), system, agent);
 
         try {
             sheet.enabled = true;
@@ -543,28 +550,27 @@ Module("styles", {
     requires: ["config", "liberator", "storage", "util"],
 
     init: function () {
-        let (array = util.Array) {
-            update(Styles.prototype, {
-                get sites() array([v.sites for ([k, v] in this.userSheets)]).flatten().uniq().__proto__,
-                completeSite: function (context, content) {
-                    context.anchored = false;
-                    try {
-                        context.fork("current", 0, this, function (context) {
-                            context.title = ["Current Site"];
-                            context.completions = [
-                                [content.location.host, "Current Host"],
-                                [content.location.href, "Current URL"]
-                            ];
-                        });
-                    }
-                    catch (e) {}
-                    context.fork("others", 0, this, function (context) {
-                        context.title = ["Site"];
-                        context.completions = [[s, ""] for (s of styles.sites)];
+        let array = util.Array;
+        update(Styles.prototype, {
+            get sites() array([v.sites for ([k, v] in this.userSheets)]).flatten().uniq().__proto__,
+            completeSite: function (context, content) {
+                context.anchored = false;
+                try {
+                    context.fork("current", 0, this, function (context) {
+                        context.title = ["Current Site"];
+                        context.completions = [
+                            [content.location.host, "Current Host"],
+                            [content.location.href, "Current URL"]
+                        ];
                     });
                 }
-            });
-        }
+                catch (e) {}
+                context.fork("others", 0, this, function (context) {
+                    context.title = ["Site"];
+                    context.completions = [[s, ""] for (s of styles.sites)];
+                });
+            }
+        });
         return storage.newObject("styles", Styles, { store: false });
     }
 }, {
