@@ -17,7 +17,7 @@ const Tabs = Module("tabs", {
     requires: ["config"],
 
     init: function () {
-        this._alternates = [config.tabbrowser.mCurrentTab, null];
+        this.updateSelectionHistory([config.tabbrowser.mCurrentTab, null]);
 
         // used for the "gb" and "gB" mappings to remember the last :buffer[!] command
         this._lastBufferSwitchArgs = "";
@@ -26,14 +26,14 @@ const Tabs = Module("tabs", {
     },
 
     _updateTabCount: function () {
-        statusline.updateTabCount(true);
+        statusline.updateField("tabcount", true);
     },
 
     _onTabSelect: function () {
         // TODO: is all of that necessary?
         //       I vote no. --Kris
         modes.reset();
-        statusline.updateTabCount(true);
+        statusline.updateField("tabcount", true);
         this.updateSelectionHistory();
         if (options["focuscontent"])
             setTimeout(function () { liberator.focusContent(true); }, 10); // just make sure, that no widget has focus
@@ -43,7 +43,10 @@ const Tabs = Module("tabs", {
      * @property {Object} The previously accessed tab or null if no tab
      *     other than the current one has been accessed.
      */
-    get alternate() this._alternates[1],
+    get alternate() {
+        var tab = this._alternates[1] ? this._alternates[1].get() : null;
+        return (tab && tab.parentNode) ? tab : null;
+    },
 
     /**
      * @property {Iterator(Object)} A genenerator that returns all browsers
@@ -58,7 +61,7 @@ const Tabs = Module("tabs", {
     /**
      * @property {number} The number of tabs in the current window.
      */
-    get count() config.tabbrowser.mTabs.length,
+    get count() config.tabbrowser.visibleTabs.length,
 
     /**
      * @property {Object} The local options store for the current tab.
@@ -112,9 +115,9 @@ const Tabs = Module("tabs", {
      */
     index: function (tab) {
         if (tab)
-            return Array.indexOf(config.tabbrowser.mTabs, tab);
+            return Array.indexOf(config.tabbrowser.visibleTabs, tab);
         else
-            return config.tabbrowser.mTabContainer.selectedIndex;
+            return Array.indexOf(config.tabbrowser.visibleTabs, config.tabbrowser.tabContainer.selectedItem);
     },
 
     // TODO: implement filter
@@ -197,7 +200,10 @@ const Tabs = Module("tabs", {
      *
      * @param {Object} tab The tab to remove.
      * @param {number} count How many tabs to remove.
-     * @param {boolean} focusLeftTab Focus the tab to the left of the removed tab.
+     * @param {number} orientation Focus orientation
+     *         1  - Focus the tab to the right of the remove tab.
+     *         0  - Focus the altanate tab of the remove tab. if alternate tab is none, same as 1
+     *         -1 - Focus the tab to the left of the remove tab.
      * @param {number} quitOnLastTab Whether to quit if the tab being
      *     deleted is the only tab in the tab list:
      *         1 - quit without saving session
@@ -205,16 +211,17 @@ const Tabs = Module("tabs", {
      * @param {boolean} force Close even if the tab is an app tab.
      */
     // FIXME: what is quitOnLastTab {1,2} all about then, eh? --djk
-    remove: function (tab, count, focusLeftTab, quitOnLastTab, force) {
+    remove: function (tab, count, orientation, quitOnLastTab, force) {
         let vTabs = config.tabbrowser.visibleTabs;
         let removeOrBlankTab = {
                 Firefox: function (tab) {
                     if (vTabs.length > 1)
                         config.tabbrowser.removeTab(tab);
                     else {
-                        if (buffer.URL != "about:blank" ||
+                        let url = buffer.URL;
+                        if (url != "about:blank" || url != "about:newtab" ||
                             window.getWebNavigation().sessionHistory.count > 0) {
-                            liberator.open("about:blank", liberator.NEW_BACKGROUND_TAB);
+                            liberator.open("", liberator.NEW_BACKGROUND_TAB);
                             config.tabbrowser.removeTab(tab);
                         }
                         else
@@ -227,18 +234,6 @@ const Tabs = Module("tabs", {
                     else
                         liberator.beep();
                 },
-                Songbird: function (tab) {
-                    if (config.tabbrowser.mTabs.length > 1)
-                        config.tabbrowser.removeTab(tab);
-                    else {
-                        if (buffer.URL != "about:blank" || window.getWebNavigation().sessionHistory.count > 0) {
-                            liberator.open("about:blank", liberator.NEW_BACKGROUND_TAB);
-                            config.tabbrowser.removeTab(tab);
-                        }
-                        else
-                            liberator.beep();
-                    }
-                }
             }[config.hostApplication] || function () {};
 
         if (typeof count != "number" || count < 1)
@@ -252,6 +247,8 @@ const Tabs = Module("tabs", {
 
             return;
         }
+        if (!orientation)
+            orientation = 0;
 
         let index = vTabs.indexOf(tab);
         // should close even if the tab is not visible such as ":tabclose arg"
@@ -266,7 +263,7 @@ const Tabs = Module("tabs", {
         }
 
         let start, end, selIndex = 0;
-        if (focusLeftTab) {
+        if (orientation < 0) {
             start = Math.max(0, index - count + 1);
             end = index;
         }
@@ -283,12 +280,22 @@ const Tabs = Module("tabs", {
                 return;
         }
 
-        if ((focusLeftTab && 0 < start - 1) || selIndex >= vTabs.length)
+        if ((orientation < 0 && 0 < start - 1) || selIndex >= vTabs.length)
             selIndex = start - 1;
 
         let currentIndex = vTabs.indexOf(tabs.getTab());
-        if (start <= currentIndex && currentIndex <= end)
-            config.tabbrowser.mTabContainer.selectedItem = vTabs[selIndex];
+        if (start <= currentIndex && currentIndex <= end) {
+            let selTab = vTabs[selIndex];
+            if (orientation == 0 &&
+                config.tabbrowser.mTabContainer.contains(this.alternate) &&
+                !this.alternate.hidden) // XXX: should be in the visible tabs ?
+            {
+                let lastTabIndex = vTabs.indexOf(this.alternate);
+                if (lastTabIndex < start || end < lastTabIndex)
+                    selTab = this.alternate;
+            }
+            config.tabbrowser.mTabContainer.selectedItem = selTab;
+        }
 
         for (let i = end; i >= start; i--) {
             removeOrBlankTab(vTabs[i]);
@@ -417,22 +424,14 @@ const Tabs = Module("tabs", {
         for (let [i, ] in tabs.browsers) {
             let index = (i + first) % nbrowsers;
             let browser = config.tabbrowser.browsers[index];
-            let url, title;
-            if ("__SS_restoreState" in browser) {
-                let entry = browser.__SS_data.entries[browser.__SS_data.index - 1];
-                url = entry.url;
-                title = entry.title || url;
-            }
-            else {
-                url = browser.contentDocument.location.href;
-                title = browser.contentDocument.title;
-            }
-            title = title.toLowerCase();
+            let tab = tabs.getTab(index);
+            let url = browser.contentDocument.location.href;
+            let title = tab.label.toLowerCase();
             if (url == buffer)
-                return [tabs.getTab(index)];
+                return [tab];
 
             if (url.indexOf(buffer) >= 0 || title.indexOf(lowerBuffer) >= 0)
-                matches.push(tabs.getTab(index));
+                matches.push(tab);
         }
 
         return matches;
@@ -519,7 +518,7 @@ const Tabs = Module("tabs", {
         if (!tab)
             tab = config.tabbrowser.mTabContainer.selectedItem;
 
-        services.get("windowWatcher")
+        services.get("ww")
                 .openWindow(window, window.getBrowserURL(), null, "chrome,dialog=no,all", tab);
     },
 
@@ -527,17 +526,10 @@ const Tabs = Module("tabs", {
      * Selects the alternate tab.
      */
     selectAlternateTab: function () {
-        liberator.assert(tabs.alternate != null && tabs.getTab() != tabs.alternate, "No alternate page");
+        let alternate = tabs.alternate;
+        liberator.assert(alternate != null && tabs.getTab() != alternate, "No alternate page");
 
-        // NOTE: this currently relies on v.tabs.index() returning the
-        // currently selected tab index when passed null
-        let index = tabs.index(tabs.alternate);
-
-        // TODO: since a tab close is more like a bdelete for us we
-        // should probably reopen the closed tab when a 'deleted'
-        // alternate is selected
-        liberator.assert(index >= 0, "Alternate buffer does not exist");  // TODO: This should read "Buffer N does not exist"
-        tabs.select(index, false, true);
+        config.tabbrowser.tabContainer.selectedItem = alternate;
     },
 
     // NOTE: when restarting a session FF selects the first tab and then the
@@ -552,7 +544,16 @@ const Tabs = Module("tabs", {
      * @see tabs#alternate
      */
     updateSelectionHistory: function (tabs) {
-        this._alternates = tabs || [this.getTab(), this._alternates[0]];
+        var tab1, tab2;
+        if (tabs && tabs.length > 1) {
+            tab1 = tabs[0] ? Cu.getWeakReference(tabs[0]) : null,
+            tab2 = tabs[1] ? Cu.getWeakReference(tabs[1]) : null;
+        }
+        else {
+            tab1 = Cu.getWeakReference(this.getTab()),
+            tab2 = this._alternates[0];
+        }
+        this._alternates = [tab1, tab2];
     }
 }, {
     copyTab: function (to, from) {
@@ -614,12 +615,19 @@ const Tabs = Module("tabs", {
                 let count   = args.count;
                 let arg     = args.literalArg;
 
+                let orientation = 1;
+                if (args["-select"] === "lastactive") {
+                    orientation = 0;
+                } else if (args["-select"] === "left") {
+                    orientation = -1;
+                }
+
                 if (arg) {
                     let removed = 0;
                     let matches = arg.match(/^(\d+):?/);
 
                     if (matches) {
-                        tabs.remove(tabs.getTab(parseInt(matches[1], 10) - 1), 1, false, 0, special);
+                        tabs.remove(tabs.getTab(parseInt(matches[1], 10) - 1), 1, orientation, 0, special);
                         removed = 1;
                     }
                     else {
@@ -641,7 +649,7 @@ const Tabs = Module("tabs", {
 
                             if (host.indexOf(str) >= 0 || uri == str ||
                                 (special && (title.indexOf(str) >= 0 || uri.indexOf(str) >= 0))) {
-                                tabs.remove(tabs.getTab(i), 1, false, 0, special);
+                                tabs.remove(tabs.getTab(i), 1, orientation, 0, special);
                                 removed++;
                             }
                         }
@@ -655,11 +663,17 @@ const Tabs = Module("tabs", {
                         liberator.echoerr("No matching tab for: " + arg);
                 }
                 else // just remove the current tab
-                    tabs.remove(tabs.getTab(), Math.max(count, 1), false, 0, special);
+                    tabs.remove(tabs.getTab(), Math.max(count, 1), orientation, 0, special);
             }, {
                 argCount: "?",
                 bang: true,
                 count: true,
+                options: [
+                    [["-select", "-s"], commands.OPTION_STRING, null,
+                        [["lastactive", "Select last active tab"],
+                         ["left", "Select the tab to the left"],
+                         ["right", "Select the tab to the right"]]],
+                ],
                 completer: function (context) completion.buffer(context),
                 literal: 0
             });
@@ -684,9 +698,13 @@ const Tabs = Module("tabs", {
         commands.add(["tab"],
             "Execute a command and tell it to output in a new tab",
             function (args) {
-                liberator.forceNewTab = true;
-                liberator.execute(args.string, null, true);
-                liberator.forceNewTab = false;
+                try {
+                    liberator.forceNewTab = true;
+                    liberator.execute(args.string, null, true);
+                }
+                finally {
+                    liberator.forceNewTab = false;
+                }
             }, {
                 argCount: "+",
                 completer: function (context) completion.ex(context),
@@ -854,7 +872,7 @@ const Tabs = Module("tabs", {
                     if (args)
                         liberator.open(args, { where: where });
                     else
-                        liberator.open("about:blank", { where: where });
+                        liberator.open("", { where: where });
                 }, {
                     bang: true,
                     completer: function (context) completion.url(context),
@@ -897,7 +915,7 @@ const Tabs = Module("tabs", {
                     liberator.assert(args.length <= 2 && !args.some(function (i) !/^\d+$/.test(i)),
                         "Trailing characters");
 
-                    let [winIndex, tabIndex] = args.map(parseInt);
+                    let [winIndex, tabIndex] = args.map(function(i) { return parseInt(i, 10) });
                     let win = liberator.windows[winIndex - 1];
 
                     liberator.assert(win, "Window " + winIndex + " does not exist");
@@ -1036,13 +1054,13 @@ const Tabs = Module("tabs", {
                 function () { tabs.list(false); commandline.show("buffers"); });
 
             mappings.add([modes.NORMAL], ["d"],
-                "Delete current buffer",
-                function (count) { tabs.remove(tabs.getTab(), count, false, 0); },
+                "Delete current tab and select the tab to the right",
+                function (count) { tabs.remove(tabs.getTab(), count, 1, 0); },
                 { count: true });
 
             mappings.add([modes.NORMAL], ["D"],
-                "Delete current buffer, focus tab to the left",
-                function (count) { tabs.remove(tabs.getTab(), count, true, 0); },
+                "Delete current buffer and select the tab to the left",
+                function (count) { tabs.remove(tabs.getTab(), count, -1, 0); },
                 { count: true });
 
             mappings.add([modes.NORMAL], ["gb"],
@@ -1108,14 +1126,13 @@ const Tabs = Module("tabs", {
                     ]
                 });
 
-            // TODO: Is this really applicable to Xulmus?
             options.add(["popups", "pps"],
                 "Where to show requested popup windows",
                 "stringlist", "tab",
                 {
                     setter: function (value) {
                         let [open, restriction] = [1, 0];
-                        for (let [, opt] in Iterator(this.parseValues(value))) {
+                        for (let opt of this.parseValues(value)) {
                             if (opt == "tab")
                                 open = 3;
                             else if (opt == "window")
