@@ -5,22 +5,20 @@
 
 /** @scope modules */
 
-//xxx:
-this.$$Namespace = function (prefix, uri) {
-    return "XMLList" in window ? Namespace(prefix, uri)
-        : {
-            prefix: prefix,
-            uri: uri,
-            toString: function () {
-                return this.uri;
-            },
+const {XHTML, XUL, NS} = (function () {
+    function Namespace(prefix, uri) {
+        this.prefix = prefix;
+        this.uri = uri;
+    }
+    Namespace.prototype.toString = function toString() { return this.uri; };
+    return {
+        XHTML: new Namespace("html", "http://www.w3.org/1999/xhtml"),
+        XUL  : new Namespace("xul", "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul"),
+        NS   : new Namespace("liberator", "http://vimperator.org/namespaces/liberator"),
     };
-}
-const XHTML = $$Namespace("html", "http://www.w3.org/1999/xhtml");
-const XUL = $$Namespace("xul", "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
-const NS =  $$Namespace("liberator", "http://vimperator.org/namespaces/liberator");
-if ("XMLList" in window) eval("default xml namespace = XHTML");
-delete this.$$Namespace;
+})();
+
+let Encoder = Components.Constructor("@mozilla.org/layout/documentEncoder;1?type=text/plain", "nsIDocumentEncoder", "init");
 
 const Util = Module("util", {
     init: function () {
@@ -140,7 +138,7 @@ const Util = Module("util", {
      * @returns {string}
      */
     escapeString: function escapeString(str, delimiter) {
-        if (delimiter == undefined)
+        if (delimiter === undefined)
             delimiter = '"';
         return delimiter + str.replace(/([\\'"])/g, "\\$1").replace("\n", "\\n", "g").replace("\t", "\\t", "g") + delimiter;
     },
@@ -260,9 +258,9 @@ const Util = Module("util", {
         zip.open(FILE, io.MODE_CREATE | io.MODE_WRONLY | io.MODE_TRUNCATE);
         function addURIEntry(file, uri)
             zip.addEntryChannel(PATH + file, TIME, 9,
-                services.get("io").newChannel(uri, null, null), false);
+                services.get("io").newChannel(uri, null, null), false)
         function addDataEntry(file, data) // Inideal to an extreme.
-            addURIEntry(file, "data:text/plain;charset=UTF-8," + encodeURI(data));
+            addURIEntry(file, "data:text/plain;charset=UTF-8," + encodeURI(data))
 
         let empty = util.Array.toObject(
             "area base basefont br col frame hr img input isindex link meta param"
@@ -285,28 +283,28 @@ const Util = Module("util", {
 
                         data.push("<"); data.push(node.localName);
                         if (node instanceof HTMLHtmlElement)
-                            data.push(" xmlns=" + XHTML.uri.quote());
+                            data.push(" xmlns=" + JSON.stringify(XHTML.uri));
 
                         for (let { name: name, value: value } in util.Array.itervalues(node.attributes)) {
-                            if (name == "liberator:highlight") {
+                            if (name === "liberator:highlight") {
                                 name = "class";
                                 value = "hl-" + value;
                             }
-                            if (name == "href") {
-                                if (value.indexOf("liberator://help-tag/") == 0)
+                            if (name === "href") {
+                                if (value.indexOf("liberator://help-tag/") === 0)
                                     value = services.get("io").newChannel(value, null, null).originalURI.path.substr(1);
                                 if (!/[#\/]/.test(value))
                                     value += ".xhtml";
                             }
-                            if (name == "src" && value.indexOf(":") > 0) {
-                                chrome[value] = value.replace(/.*\//, "");;
+                            if (name === "src" && value.indexOf(":") > 0) {
+                                chrome[value] = value.replace(/.*\//, "");
                                 value = value.replace(/.*\//, "");
                             }
                             data.push(" ");
                             data.push(name);
                             data.push('="');
                             data.push(xml`${value}`.toString());
-                            data.push('"')
+                            data.push('"');
                         }
                         if (node.localName in empty)
                             data.push(" />");
@@ -315,7 +313,9 @@ const Util = Module("util", {
                             if (node instanceof HTMLHeadElement)
                                 data.push(`<link rel="stylesheet" type="text/css" href="help.css"/>`/*toXMLString()*/);
                             Array.map(node.childNodes, arguments.callee);
-                            data.push("</"); data.push(node.localName); data.push(">");
+                            data.push("</");
+                            data.push(node.localName);
+                            data.push(">");
                         }
                         break;
                     case Node.TEXT_NODE:
@@ -359,7 +359,7 @@ const Util = Module("util", {
             xmlhttp.mozBackgroundRequest = true;
             if (callback) {
                 xmlhttp.onreadystatechange = function () {
-                    if (xmlhttp.readyState == 4)
+                    if (xmlhttp.readyState === 4)
                         callback(xmlhttp);
                 };
             }
@@ -406,6 +406,20 @@ const Util = Module("util", {
             null
         );
 
+        if (services.get("vc").compare(Application.version, "33") >= 0
+            && Cu.isXrayWrapper(result)) {
+            let xr = result;
+
+            if (asIterator) {
+                result = { iterateNext: function iterateNext() xr.iterateNext()};
+            } else {
+                result = {
+                    snapshotItem: function snapshotItem(num) xr.snapshotItem(num),
+                    get snapshotLength() xr.snapshotLength,
+                };
+            }
+        }
+
         result.__iterator__ = asIterator
                             ? function () { let elem; while ((elem = this.iterateNext())) yield elem; }
                             : function () { for (let i = 0; i < this.snapshotLength; i++) yield this.snapshotItem(i); };
@@ -502,7 +516,7 @@ const Util = Module("util", {
         ]);
         if (object instanceof Element) {
             let elem = object;
-            if (elem.nodeType == elem.TEXT_NODE)
+            if (elem.nodeType === elem.TEXT_NODE)
                 return elem.data;
             function namespaced(node) {
                 var ns = NAMESPACES[node.namespaceURI];
@@ -546,7 +560,7 @@ const Util = Module("util", {
                 }
                 catch (e) {}
                 if (!hasValue) {
-                    if (i instanceof Array && i.length == 2)
+                    if (i instanceof Array && i.length === 2)
                         [i, value] = i;
                     else
                         var noVal = true;
@@ -723,9 +737,6 @@ const Util = Module("util", {
      * @returns {Node|DocumentFragment}
      */
     xmlToDom: function xmlToDom(node, doc, nodes) {
-        if (typeof node === "xml")
-            return this.xmlToDomForE4X(node, doc, nodes);
-
         var dom = this.xmlToDomForTemplate(node, doc, nodes);
 
         //xxx: change highlight's namespace
@@ -739,39 +750,6 @@ const Util = Module("util", {
         }
         return dom;
     },
-    /**
-     * Converts an E4X XML literal to a DOM node.
-     *
-     * @param {Node} node
-     * @param {Document} doc
-     * @param {Object} nodes
-     * @returns {Node}
-     * @deprecated
-     * @see util.xmlToDom
-     */
-    xmlToDomForE4X: function xmlToDomForE4X(node, doc, nodes) {
-        if (node.length() != 1) {
-            let domnode = doc.createDocumentFragment();
-            for each (let child in node)
-                domnode.appendChild(arguments.callee(child, doc, nodes));
-            return domnode;
-        }
-        switch (node.nodeKind()) {
-        case "text":
-            return doc.createTextNode(String(node));
-        case "element":
-            let domnode = doc.createElementNS(node.namespace(), node.localName());
-            for each (let attr in node.attributes())
-                domnode.setAttributeNS(attr.name() == "highlight" ? NS.uri : attr.namespace(), attr.name(), String(attr));
-            for each (let child in node.children())
-                domnode.appendChild(arguments.callee(child, doc, nodes));
-            if (nodes && node.attribute("key"))
-                nodes[node.attribute("key")] = domnode;
-            return domnode;
-        default:
-            return null;
-        }
-    },
     /**
      * Converts an string of TemplateXML object to a DOM node.
      *
@@ -805,8 +783,7 @@ const Util = Module("util", {
      * @param {Number} flags    nsIDocumentEncoder.OutputXXXX
      * @returns {String}
      */
-    domToStr: let (Encoder = Components.Constructor("@mozilla.org/layout/documentEncoder;1?type=text/plain", "nsIDocumentEncoder", "init"))
-    function domToStr(node, type, flags) {
+    domToStr: function domToStr(node, type, flags) {
         var doc, method;
 
         if (node instanceof Document) {
@@ -864,7 +841,7 @@ const Util = Module("util", {
         }
     }, {
         isinstance: function isinstance(obj) {
-            return Object.prototype.toString.call(obj) == "[object Array]";
+            return Object.prototype.toString.call(obj) === "[object Array]";
         },
 
         /**
